PROGRAM RedOrBlack ;

USES DOS, CRT, JPDoor30, StrSup ;

CONST
   Version  = '1.0' ;
   BlackRow = 8 ;         { black hand's screen row number }
   RedRow   = 13 ;        { red hand's screen row number   }
                          { card column numbers:           }
   CardCol  : ARRAY [1..20] OF BYTE = (16,18,21,24,27,30,33,36,39,42,
                                       45,48,51,54,57,60,63,66,69,72) ;

TYPE
   DeckRec  = RECORD                   { card structure:               }
                 CardNum : STR2 ;      { values: 'A'..'13' A is low    }
                 Suit    : CardSuit ;  { use JPDoor CardSuit (C,D,H,S) }
              END ;

   PlayRec  = RECORD                                 { player's data         }
                 UserName  : STRING[35] ;            { player's name         }
                 LastDate  : STRING[18] ;            { last date played      }
                 UserScore : LONGINT ;               { player's score        }
                 UserTurns : INTEGER ;               {player's turn count as
                                                      of lastdate            }
                 UserTotal : LONGINT ;               {same as userturns (spare
                                                      data)                  }
                 PArray    : Array [1..31] OF CHAR ; {days played this month
                                                      '-' = not played,
                                                      'X' = played           }
                 MakeUps   : INTEGER ;               { make-ups used on date
                                                       LastDate              }
                 Speed     : INTEGER ;               { player's delay setting}
              END ;

   HighRec  = RECORD                                 { high score record     }
                 PlayerName     : STRING[35] ;       { player name           }
                 Score          : LONGINT ;          { player score          }
              END ;

   MonthRec = RECORD                                 { .DTE month record     }
                 MonthVal: WORD ;                    { current month number  }
              END ;


VAR
   yr,mn,dt,dy   : WORD ;                            { getdate globals       }
   CardDeck      : ARRAY [1..312] OF DeckRec ;       { card deck array       }
   Card          : DeckRec ;                         { card deck record      }
   FirstCard     : DeckRec ;                         { first card record     }
   DropPath      : STRING ;                          { path to dropfile      }
   Graphics,                                         { player's setting      }
   Holding       : CHAR ;                            { global character var  }
   Score         : LONGINT ;                         { global score var      }
   TurnCount     : INTEGER ;                         { global turncount var  }
   HighFile      : FILE OF HighRec ;                 { high score file       }
   High          : HighRec ;                         { high score record     }
   UserFile      : FILE OF PlayRec ;                 { player data file      }
   User          : PlayRec ;                         { player data record    }
   MonthFile     : FILE OF MonthRec ;                { month data file       }
   Month         : MonthRec ;                        { month data record     }
   MaxTurns      : INTEGER ;                         { max turns per day var }
   AbortGame     : BOOLEAN ;                         { global boolean if game
                                                       is aborted            }
   FirstPlay     : BOOLEAN ;                         { is this the player's
                                                       first hand?           }
   MaxMakeUpDays : INTEGER ;                         { max makeup days var   }
   PlayArray     : ARRAY [1..31] OF CHAR ;           { global array of days
                                                       player has made up or
                                                       has available         }
   UserMakeUps   : INTEGER ;                         { global of how many make
                                                       ups user has used for
                                                       LastDate              }
   GameStarted   : BOOLEAN ;                         { have data files been
                                                       opened? Default=false }
   MaxBet        : LONGINT ;                         { maximum wager var     }
   BetAmount     : LONGINT ;                         { player's current wager}
   BetType       : CHAR ;                            { what did player bet on}
   CardCounter,                                      { global card counter   }
   RedScore,                                         { global red hand score }
   BlackScore,                                       { glob. black hand score}
   BlackCounter,                                     { glob. black card count}
   RedCounter    : INTEGER ;                         { glob. red card count  }
   BlackDiff,                                        { glob black difference }
   RedDiff       : INTEGER ;                         { glob red difference   }
   DSpeed        : INTEGER ;                         { global delay speed    }
   LoopVar       : INTEGER ;                         { global loop variable  }


PROCEDURE JPDelay(n:INTEGER ; KeyGet,EatChar : BOOLEAN) ;
{  this procedure uses the DOS second timer to cause a delay
   of 'n' seconds.

   KeyGet   : if set to true then if a key is pressed during the
              delay, it will be retrieved.
   EatChar  : if set to true, if a key is pressed during the
              delay, and KeyGet is set to true, then the character
              pressed, will be discarded or "eaten".
}
VAR
   hr,mn,sec,sec100    : WORD ;
   sectemp             : INTEGER ;
   counter             : INTEGER ;
BEGIN
   GetTime(hr,mn,sec,sec100) ;
   sectemp := sec ;
   counter := 0 ;
   REPEAT
      GetTime(hr,mn,sec,sec100) ;
      IF sec <> sectemp THEN BEGIN
         sectemp := sec ;
         INC(counter) ;
      END ;
      IF KeyGet THEN BEGIN
         IF (NOT Local) AND (InReady) THEN BEGIN
            IF EatChar THEN holding := GetChar ;
            counter := n + 1 ;
         END ELSE IF KeyPressed THEN BEGIN
            IF EatChar THEN holding := READKEY ;
            counter := n + 1 ;
         END ;
      END ;
   UNTIL counter >= n ;
END ;


FUNCTION AtoI(Tstr:STRING) : LONGINT ;
{  Convert a string 'Tstr' to a long integer
   value
}
VAR
   num        : LONGINT ;
   code       : INTEGER ;
BEGIN
   VAL(Tstr,num,code) ;
   IF code <> 0 THEN BEGIN
      AtoI := 0 ;
      EXIT ;
   END ELSE BEGIN
      AtoI := num ;
      EXIT ;
   END ;
END ;

PROCEDURE GetParms ;
{ get the command line parameters passed to the program and
  act on them accordingly.
}
VAR
   num,
   code     : INTEGER ;
   Prefix   : STRING ;                       { parameter prefix             }
   Suffix   : INTEGER ;                      { parameter suffix             }
   jj       : BYTE ;                         { local loop counter           }
BEGIN
   MaxTurns := 5 ;                           { set default MaxTurns         }
   MaxMakeUpDays := 3 ;                      { set default MaxMakeUpDays    }
   MaxBet := 10000 ;                         { set default MaxBet           }
   DoorSys := FALSE;                         { set default DoorSys (JPDoor) }
   IF ParamCount = 0 THEN BEGIN
     DropPath := '' ;                        { set droppath to current DIR  }
     EXIT ;
   END ;
   FOR jj := 1 TO ParamCount DO BEGIN
      Prefix := ChangeCase(COPY(ParamStr(jj),1,2)) ;
      Suffix := AtoI(COPY(ParamStr(jj),3,69)) ;
      IF Prefix = '/T' THEN BEGIN            { get MaxTurns                 }
         IF (Suffix > 0) AND (Suffix < 100) THEN MaxTurns := Suffix ;
      END ELSE IF Prefix = '/M' THEN BEGIN   { get MaxMakeUpDays            }
         IF (Suffix > 0) AND (Suffix < 6) THEN MaxMakeUpDays := Suffix ;
      END ELSE IF Prefix = '/B' THEN BEGIN   { get MaxBet                   }
         IF (Suffix > 0) AND (Suffix < 500001) THEN MaxBet := Suffix ;
      END ELSE IF Prefix = '/D' THEN BEGIN   { set DoorSys to TRUE          }
         DoorSys := TRUE ;
      END ELSE IF Prefix = '/P' THEN BEGIN   { get droppath                 }
         DropPath := CkPath(COPY(ParamStr(jj),3,65)) ;
      END ;
   END ;
END ;

(*  The following procedures use RECORD locking on their associated files. *)
(*  This is done using the JPDoor routines Lock and UnLock.  Using this    *)
(*  method, along with a Turbo Pascal FileMode variable setting of 66,     *)
(*  ensures the lock is done only on a per-record basis and permits you    *)
(*  to write stable multiline door programs.                               *)


PROCEDURE WriteUser;
{  this procedure writes to the player data file using record
   locking routines Lock and UnLock found in JPDoor.  These
   routines utilize the DOS SHARE facility
}
VAR
   LockStart,
   LockEnd     : Longint ;
   Err,
   ErrCnt      : Integer;
BEGIN
   ErrCnt := 0;
   LockStart := (filepos(userfile) * Sizeof(user)) + 1 ;
   LockEnd := (filepos(userfile) + 1) * sizeof(user) ;
   REPEAT
      Inc(ErrCnt) ;
      Err := Lock(userfile,lockstart,lockend) ;
   UNTIL (err = 0) or (ErrCnt > 9999) ;
   IF ErrCnt < 10000 THEN BEGIN
      Write(userfile,user) ;
      UnLock(userfile,LockStart,LockEnd) ;
   END ;
END ;

PROCEDURE ReadUser;
{  this procedure reads from the player data file using record
   locking routines Lock and UnLock found in JPDoor.  These
   routines utilize the DOS SHARE facility
}
VAR
   LockStart,
   LockEnd     : LongInt ;
   Err,
   ErrCnt      : Integer;
BEGIN
   ErrCnt := 0;
   LockStart := (filepos(userfile) * Sizeof(user)) + 1 ;
   LockEnd := (filepos(userfile) + 1) * sizeof(user) ;
   REPEAT
      Inc(ErrCnt) ;
      Err := Lock(userfile,lockstart,lockend) ;
   UNTIL (err = 0) or (ErrCnt > 9999) ;
   IF ErrCnt < 10000 THEN BEGIN
      Read(UserFile,User);
      UnLock(UserFile,LockStart,LockEnd) ;
   END ;
END ;

PROCEDURE WriteHighScore;
{ this procedure writes to the high score file using record
  locking routines Lock and UnLock found in JPDoor
}
VAR
   i,
   Err,
   ErrCnt      : Integer;
BEGIN
   ErrCnt := 0;
   Err := 1;
   i := FilePos(HighFile);
   WHILE (Err <> 0) AND (ErrCnt < 10000) DO BEGIN
      Inc(ErrCnt);
      Err := Lock(HighFile,(I*SizeOf(High))+1,((I+1)*SizeOf(High)));
   END ;
   IF ErrCnt < 9999 THEN BEGIN
      Write(HighFile,High);
      UnLock(HighFile,(I*SizeOf(High))+1,((I+1)*SizeOf(High)));
   END ;
END ;

PROCEDURE ReadHighScore;
{ this procedure reads from the high score file using record
  locking routines Lock and UnLock found in JPDoor.  You can
  see that two slightly different methods have been used
  throughout the reading/writing of records.
}
VAR
   LockStart,
   LockEnd     : Longint ;
   Err,
   ErrCnt      : Integer;
BEGIN
   ErrCnt := 0;
   LockStart := (filepos(highfile) * Sizeof(high)) + 1 ;
   LockEnd := (filepos(highfile) + 1) * sizeof(high) ;
   REPEAT
      Inc(ErrCnt) ;
      Err := Lock(highfile,lockstart,lockend) ;
   UNTIL (err = 0) or (ErrCnt > 9999) ;
   IF ErrCnt < 10000 THEN BEGIN
      Read(HighFile,High);
      UnLock(HighFile,LockStart,LockEnd);
   END ;
END ;


PROCEDURE UpdatePlayer ;
{ this procedure is called upon program termination to update
  the online player's data.  Global variables are used to track
  the player's data throughout game play.  Only at program
  termination do these variables get written to the user's
  data record in the data file.
}
VAR
   jj     : BYTE ;
BEGIN
   ASSIGN(UserFile,'Rob.Dat') ;
   {$I-} RESET(UserFile) ; {$I+}
   IF IOResult <> 0 THEN EXIT ;
   WHILE NOT EOF(UserFile) DO BEGIN
      READUser;
      IF User.UserName = UserName THEN BEGIN
         FOR jj := 1 TO 31 DO User.PArray[jj] := PlayArray[jj] ;
         User.MakeUps := UserMakeUps ;
         User.LastDate := Date ;
         User.UserScore := Score ;
         User.UserTurns := TurnCount ;
         User.UserTotal := Score ;
         User.Speed := DSpeed ;
         SEEK(UserFile,FilePos(UserFile)-1) ;
         WriteUser;
         SEEK(UserFile,FileSize(UserFile)) ;
      END ;
   END ;
   CLOSE(UserFile) ;
END ;


PROCEDURE CloseGame ;
{ this procedure is called by the Terminate procedure when it
  is appropriate.  If needed, player data is updated.  Status
  bar is turned off and the screen cleared......
}
BEGIN
   IF GameStarted THEN UpdatePlayer ;
   StatusLineOn := FALSE ;
   SetColor(0,7,0) ;
   ClearScreen ;
   WRITELN(' Closing game files') ;
END ;


PROCEDURE LastComments ;
{ This procedure contains a variety of little things to be
  displayed to the screen upon normal program termination.
  This procedure is not called if the user is not actually
  still online (eg: user dropped carrier, etc.).  The last
  foreground color displayed should be gray (7) and the
  last background should be black (0) so that the SysOp's
  DOS color will be reset
}
BEGIN
   sDisplay(0,14,0,'This version of ') ;
   sDisplay(0,11,0,'Red Or Black ') ;
   sDisplay(0,14,0,'is ') ;
   sDisplay(0,12,1,'PUBLIC DOMAIN') ;
   Display(0,7,0,' ') ;  CRLF ;
   CRLF ;
   sDisplay(0,14,0,'Closing the ') ;
   sDisplay(0,12,0,'Red Or Black ') ;
   sDisplay(0,14,0,'door and returning to ') ;
   sDisplay(0,11,0,FormatStr(SystemName)) ;
   Display(0,14,0,'...') ;
   CRLF ;
   Display(0,7,0,'One moment...') ;
   CRLF ;
END ;


PROCEDURE Terminate(n:byte) ;
{ This procedure is called whenever a program exit (halt) is
  encountered.  You can see here where JPDoor passes the
  halt number in 'n' depending on the type of program
  termination.  See your JPDoor manual for a list of halt codes
  used by JPDoor.
}
BEGIN
   CASE n OF
      0 : BEGIN
             CloseGame ;
             LastComments ;
          END ;
      1 : BEGIN
             Writeln(' Carrier lost') ;
             CloseGame ;
          END ;
      2 : BEGIN
             Writeln(' Time limit exceeded') ;
             CloseGame ;
             LastComments ;
          END ;
      3 : BEGIN
             Writeln(' User inactive') ;
             CloseGame ;
             LastComments ;
          END ;
      4 : BEGIN
             IF NOT DoorSys THEN
                Writeln(' Dorinfo'+Node+'.Def file not found')
             ELSE Writeln(' Door.Sys file not found') ;
          END ;
      5 : BEGIN
             Writeln('EXITINFO.BBS or DOOR.SYS file not found') ;
          END ;
      6 : BEGIN
             Writeln(' Directory read/change error') ;
          END ;
      7 : BEGIN
             Writeln(' CTS timeout error') ;
             CloseGame ;
          END ;
      8 : BEGIN  {null exit - like no ansi enabled}
             StatusLineOn := FALSE ;
             LastComments ;
          END ;
      15: BEGIN  {null exit - like the program never got started }
          END ;
      ELSE CloseGame ;  { probable runtime error. check Rob.log }
   END ;
END ;

{ The following two lines include the source files ROBEXIT.PAS
  and ROBSORT.PAS which are the TrapExit and Sort routines
}

{$I RobExit}
{$I RobSort}


PROCEDURE GetUserInfo ;
{ This procedure is called in the main body of the program to retrieve
  the player's data or to setup the player as a newcomer to the game
}
VAR
   PlayerFound    : BOOLEAN ;
   jj             : BYTE ;
BEGIN
   PlayerFound := FALSE ;
   ASSIGN(UserFile,'ROB.DAT') ;
   {$I-} RESET(UserFile) ; {$I+}               { if the data file isn't there}
   IF IORESULT <> 0 THEN REWRITE(UserFile) ;   { create the file             }
   WHILE NOT EOF(UserFile) DO BEGIN
      READUser;                                { read record with locking    }
      IF User.UserName = UserName THEN BEGIN   { setup global variables      }

         FOR jj := 1 TO 31 DO PlayArray[jj] := User.PArray[jj] ;
         DSpeed := User.Speed ;
         IF (User.UserScore < 200) AND (User.LastDate <> Date) THEN BEGIN
            { user has a lower-than-starting level score so give them 1,000 }
            Display(0,7,0,'Since you have less than $200, '
                          +FormatStr(SystemName)) ;
            Display(0,7,0,'has agreed to start you off with another $1,000.') ;
            Crlf ;
            More('Press [any key] to continue',12) ;
            Score := 1000 ;
         END ELSE Score := User.UserScore ;
         IF User.LastDate = Date THEN BEGIN    { if user last called today.. }
            TurnCount := User.UserTurns ;
            UserMakeUps := User.MakeUps ;
         END ELSE BEGIN                        { otherwise it's a new day    }
            TurnCount := 0 ;
            UserMakeUps := 0 ;
         END ;
         PlayerFound := TRUE ;
         SEEK(UserFile,FileSize(UserFile)) ;
      END ;
   END ; { while }
   IF NOT PlayerFound THEN BEGIN    { player is new                 }
      CRLF ;
      Display(0,7,0,'Greetings new player!') ;
      CRLF ;
      FOR jj := 1 TO 31 DO BEGIN    { initialize player and global  }
         User.PArray[jj] := '-' ;   { make up day array             }
         PlayArray[jj] := '-' ;
      END ;
      User.Speed := 1000 ;          { initialize player's variables }
      DSpeed := 1000 ;
      User.MakeUps := 0 ;
      UserMakeUps := 0 ;
      User.UserName := UserName ;
      User.UserScore := 1000 ;
      User.UserTotal := 1000 ;
      Score := 1000 ;
      User.LastDate := Date ;
      User.UserTurns := 0 ;
      TurnCount := 0 ;
      SEEK(UserFile,FileSize(UserFile)) ;
      WRITEUser;                  { write player data with locking }
   END ;
   CLOSE(UserFile) ;              { close the data file when done  }
END ;


PROCEDURE CheckMonth ;
{  This procedure is called to check to see if the month has
   changed.  If it has, then it's time to start the game
   over and to create a high score file for last month.
}
VAR
   LoopVar     : BYTE ;
   StopPoint   : BYTE ;
   yr,mn,dt,dy : WORD ;
   TextFile    : TEXT ;
BEGIN
   ASSIGN(MonthFile,'ROB.DTE') ;
   {$I-} RESET(MonthFile) ; {$I+}
   IF IOResult <> 0 THEN BEGIN         { if the month file doesn't exist...}
      REWRITE(MonthFile) ;             { create it and store an invalid    }
      Month.MonthVal := 14 ;           { month number 14                   }
      WRITE(MonthFile,Month) ;         { write it to the month file        }
      RESET(MonthFile) ;               { reset to top of file              }
      IF EXIST('Rob.Hsc') THEN BEGIN   { if a high score file exists then  }
         ASSIGN(HighFile,'Rob.Hsc') ;  { assign it and erase it            }
         ERASE(HighFile) ;
      END ;
   END ;
   RESET(MonthFile) ;
   READ(MonthFile,Month) ;                { read the current month data       }
   CLOSE(MonthFile) ;                     { close month file when done        }
   GetDate(yr,mn,dt,dy) ;                 { get the current date              }
   IF Month.MonthVal <> mn THEN BEGIN     { are the months different? if so   }
       RESET(MonthFile) ;                 { reset to top of file (open file)  }
       Month.MonthVal := mn ;             { store current month number        }
       WRITE(MonthFile,Month) ;           { and write it to disk              }
       CLOSE(MonthFile) ;                 { close month file when done        }
       IF NOT EXIST('Rob.Dat') THEN EXIT ;{ if there's no data file, exit now }
       Display(0,7,0,'Initializing new month...') ;
       DELAY(1000) ;
       ASSIGN(UserFile,'Rob.Dat') ;
       RESET(UserFile) ;                  { open the player data file         }
       IF FileSize(UserFile) < 1 THEN BEGIN  { if data file is empty then     }
          CLOSE(UserFile) ;                  { close and exit                 }
          EXIT ;
       END ;
       IF FileSize(UserFile) < 3 THEN StopPoint := FileSize(UserFile)
         ELSE StopPoint := 3 ;
       ASSIGN(HighFile,'Rob.Hsc') ;
       REWRITE(HighFile) ;
       FOR LoopVar := 1 TO StopPoint DO BEGIN
          READUser;
          High.PlayerName := User.UserName ;
          High.Score := User.UserScore ;
          WRITEHighScore;
       END ;
       CLOSE(UserFile) ;
       ERASE(UserFile) ;
       CLOSE(HighFile) ;
    END ;
END ;

 PROCEDURE ViewScores(n:BYTE) ;

 { n:  1=view top scores at end of game  2=view all of player list }

 VAR
    Lc          : INTEGER ;
    yn          : CHAR ;
    TempVal     : BYTE ;
    FinS        : STRING ;
 BEGIN
    SetColor(0,7,0) ;
    ClearScreen ;
    IF n = 1 THEN BEGIN
       Display(0,15,0,'One moment...') ;
       RobSort ;                          { sort the player data file         }
       ClearScreen ;
    END ;
    Lc := 0 ;
    ASSIGN(UserFile,'Rob.Dat') ;
    RESET(UserFile) ;
    ClearScreen ;
    IF n = 2 THEN
    sDisplay(1,15,0,' PLAYER ROSTER                                                                ')
    ELSE
    sDisplay(1,15,0,' TOP 15 PLAYERS                                                               ') ;
    sDisplay(0,7,0,' ') ;
    CRLF ; CRLF ;
    Display(0,13,0,MkString(17,' ')+'Player Name                         Score') ;
     Display(0,7,0,MkString(17,' ')+'---------------------------------   ----------') ;
    IF n = 2 THEN SetHotKeys_ON(NoKill,['S']) ;
    WHILE NOT EOF(UserFile) DO BEGIN
       READUser;
       INC(Lc) ;

       IF User.UserName = UserName THEN
          Display(0,11,0,MkString(17,' ')+LJust(FormatStr(User.UserName),36) + RJust(ItoA(Score),10))
          ELSE
          Display(0,11,0,MkString(17,' ')+LJust(FormatStr(User.UserName),36) + RJust(ItoA(User.UserScore),10)) ;

       IF n = 1 THEN BEGIN
          IF Lc = 15 THEN SEEK(UserFile,FileSize(UserFile)) ;
       END ELSE BEGIN
          IF Lc MOD 24 = 0 THEN More('Press [any key] to continue',12) ;
       END ;
    END ;
    CLOSE(UserFile) ;
    CRLF ;
    sDisplay(0,12,0,Center('Press [any key] to continue',79)) ;
    holding := getchar ;
    IF n = 2 THEN SetHotKeys_OFF ;
 END ;

PROCEDURE ClearCards ;
CONST
   RowNumber : ARRAY [1..6] OF BYTE = (8,9,10,13,14,15) ;
VAR
   jj        : BYTE ;
BEGIN
   FOR jj := 1 TO 6 DO BEGIN
      CursorPos(RowNumber[jj],16) ;
      sDisplay(0,15,0,#27+'[K') ;
   END ;
END ;

PROCEDURE ClearBlackScore ;
BEGIN
   CursorPos(9,6) ;
   sDisplay(0,14,0,'  ') ;
END ;

PROCEDURE ClearRedScore ;
BEGIN
   CursorPos(14,6) ;
   sDisplay(0,14,0,'  ') ;
END ;

PROCEDURE ClearBet ;
BEGIN
   CursorPos(18,71) ;
   sDisplay(2,15,0,MkString(8,' ')) ;
END ;

PROCEDURE ClearResult ;
BEGIN
   CursorPos(19,71) ;
   sDisplay(6,15,0,'        ') ;
END ;

PROCEDURE ClearFirstCard ;
BEGIN
   CursorPos(19,22) ;
   sDisplay(6,15,0,'   ') ;
END ;

PROCEDURE WriteBet ;
BEGIN
   CursorPos(18,71) ;
   sDisplay(2,15,0,RJust('$'+ItoA(BetAmount),8)) ;
END ;

PROCEDURE WriteBlackScore(n : INTEGER) ;
BEGIN
   CursorPos(9,6) ;
   sDisplay(0,14,0,RJust(ItoA(n),2)) ;
END ;

PROCEDURE WriteRedScore(n : INTEGER) ;
BEGIN
   CursorPos(14,6) ;
   sDisplay(0,14,0,RJust(ItoA(n),2)) ;
END ;


PROCEDURE WriteFirstCard ;
VAR
   Tc    : STRING[1] ;
BEGIN
   CASE ORD(FirstCard.Suit) OF
      0  : Tc := clubchar ;  {clubs}
      1  : Tc := #4 ;        {diamonds}
      2  : Tc := #3 ;        {hearts}
      3  : Tc := #6 ;        {spades}
   END ;
   CursorPos(19,22) ;
   sDisplay(6,15,0,RJust(FirstCard.CardNum+Tc,3)) ;
END ;

PROCEDURE WriteResult(tstr:STRING) ; {write color or inverse}
BEGIN
   CursorPos(19,71) ;
   sDisplay(6,15,0,'        ') ;
   CursorPos(19,71) ;
   sDisplay(6,15,0,RJust(tstr,8)) ;
END ;

PROCEDURE WriteScore ;
BEGIN
   CursorPos(17,68) ;
   sDisplay(5,15,0,RJust('$'+ItoA(Score),11)) ;
END ;

PROCEDURE WriteMaxTurns ;
BEGIN
   CursorPos(18,23) ;
   sDisplay(2,15,0,RJust(ItoA(MaxTurns),2)) ;
END ;

PROCEDURE WriteTurnCount ;
BEGIN
   CursorPos(17,23) ;
   sDisplay(5,15,0,RJust(ItoA(TurnCount),2)) ;
END ;

PROCEDURE GoCmd ;
BEGIN
   CursorPos(19,53) ;
END ;


PROCEDURE ShowMaxTurns ;
BEGIN
   CRLF ;
   Display(0,7,0,'You have already used your turns for today!') ;
   Display(0,7,0,'Be sure to come back again tomorrow to see how you do.') ;
   CRLF ;
   More('Press [any key] to continue',12) ;
   ViewScores(1) ;
   HALT(0) ;
END ;

PROCEDURE WriteMsg(instr:STRING) ;
BEGIN
   CursorPos(21,1) ;
   sDisplay(0,15,0,#27+'[K'+instr) ;
END ;

PROCEDURE ClearMenu ;
BEGIN
   CursorPos(17,26) ;
   sDisplay(0,15,0,MkString(29,' ')) ;
   CursorPos(18,26) ;
   sDisplay(0,15,0,MkString(29,' ')) ;
   CursorPos(19,26) ;
   sDisplay(0,15,0,MkString(29,' ')) ;
END ;



PROCEDURE WriteMenu ;
BEGIN
   CursorPos(17,29) ;
   sDisplay(0,13,0,'D') ;
   sDisplay(0,15,0,'>elay Speed    ') ;
   sDisplay(0,13,0,'P') ;
   sDisplay(0,15,0,'>lay') ;
   CursorPos(18,29) ;
   sDisplay(0,13,0,'H') ;
   sDisplay(0,15,0,'>elp           ') ;
   sDisplay(0,13,0,'Q') ;
   sDisplay(0,15,0,'>uit') ;
   CursorPos(19,29) ;
   sDisplay(0,13,0,'V') ;
   sDisplay(0,15,0,'>iew Scores    ') ;
   sDisplay(0,14,0,'Select:  ') ;
   GoCmd ;
END ;




PROCEDURE ShuffleDeck ;

VAR
   loopvar : INTEGER ;
   counter : INTEGER ;
   r       : INTEGER ;
   Success : BOOLEAN ;
   NewDeck : ARRAY [1..312] OF DeckRec ;
BEGIN
   RANDOMIZE ;
   counter := 0 ;
   IF (CardCounter < 212) AND (CardCounter > 0) THEN EXIT ;
   WriteMsg('Shuffling 6 decks of 52 cards each...') ;
   DELAY(1000) ;
   FOR loopvar := 1 TO 6 DO BEGIN
      INC(counter) ;
      NewDeck[counter].CardNum := 'A' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'A' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'A' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'A' ;
      NewDeck[counter].Suit := S ;
      INC(counter) ;
      NewDeck[counter].CardNum := '2' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := '2' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := '2' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := '2' ;
      NewDeck[counter].Suit := S ;
      INC(counter) ;
      NewDeck[counter].CardNum := '3' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := '3' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := '3' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := '3' ;
      NewDeck[counter].Suit := S ;
      INC(counter) ;
      NewDeck[counter].CardNum := '4' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := '4' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := '4' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := '4' ;
      NewDeck[counter].Suit := S ;
      INC(counter) ;
      NewDeck[counter].CardNum := '5' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := '5' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := '5' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := '5' ;
      NewDeck[counter].Suit := S ;
      INC(counter) ;
      NewDeck[counter].CardNum := '6' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := '6' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := '6' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := '6' ;
      NewDeck[counter].Suit := S ;
      INC(counter) ;
      NewDeck[counter].CardNum := '7' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := '7' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := '7' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := '7' ;
      NewDeck[counter].Suit := S ;
      INC(counter) ;
      NewDeck[counter].CardNum := '8' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := '8' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := '8' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := '8' ;
      NewDeck[counter].Suit := S ;
      INC(counter) ;
      NewDeck[counter].CardNum := '9' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := '9' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := '9' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := '9' ;
      NewDeck[counter].Suit := S ;
      INC(counter) ;
      NewDeck[counter].CardNum := '10' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := '10' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := '10' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := '10' ;
      NewDeck[counter].Suit := S ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'J' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'J' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'J' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'J' ;
      NewDeck[counter].Suit := S ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'Q' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'Q' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'Q' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'Q' ;
      NewDeck[counter].Suit := S ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'K' ;
      NewDeck[counter].Suit := C ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'K' ;
      NewDeck[counter].Suit := D ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'K' ;
      NewDeck[counter].Suit := H ;
      INC(counter) ;
      NewDeck[counter].CardNum := 'K' ;
      NewDeck[counter].Suit := S ;
   END ; {6 decks of 52 now in NewDeck}
   Success := FALSE ;
   FOR loopvar := 1 TO 312 DO BEGIN    {shuffle the initialized deck}
      REPEAT
         REPEAT
            r := random(313) ;
         UNTIL (r > 0) AND (r < 313) ;
         IF NewDeck[r].CardNum <> 'U' THEN BEGIN
            CardDeck[loopvar].CardNum := NewDeck[r].CardNum ;
            CardDeck[loopvar].Suit := NewDeck[r].Suit ;
            NewDeck[r].CardNum := 'U' ;
            Success := TRUE ;
         END ELSE Success := FALSE ;
      UNTIL Success ;
   END ;
   sDisplay(0,15,0,'done!') ;
   CardCounter := 0 ;
END ;

FUNCTION CardVal(instr : STR2) : INTEGER ;
{ returns the value of the card number passed to
  this function as a String[2].
}
VAR
   TVal   : INTEGER ;
BEGIN
   IF instr = 'A' THEN TVal := 1          { if it's an ACE...                }
                                          { if it's a face card then....     }
   ELSE IF (instr = 'K') OR (instr = 'Q') OR (instr = 'J') THEN TVal := 10
   ELSE TVal := AtoI(instr) ;             { otherwise, assign its face value }
   CardVal := TVal ;
END ;

PROCEDURE GetDiffs ;
{ set the differences between hands for the
  fuzzy logic below.  Sets global variables
  BlackDiff and RedDiff.
}
BEGIN
   IF BlackScore = 31 THEN BlackDiff := 0
     ELSE BlackDiff := BlackScore - 31 ;
   IF RedScore = 31 THEN RedDiff := 0
     ELSE RedDiff := RedScore - 31 ;
END ;

FUNCTION Half(n:LONGINT) : LONGINT ;
{ return half the value passed }
BEGIN
   Half := Trunc((n DIV 2)) ;
   Exit ;
END ;


PROCEDURE DealCards ;
{ deal the cards during play }
BEGIN
   INC(CardCounter) ;
   FirstCard.CardNum := CardDeck[CardCounter].CardNum ;
   FirstCard.Suit := CardDeck[CardCounter].Suit ;
   WriteFirstCard ;

   ClearMenu ;
   CursorPos(18,28) ;
   sDisplay(0,12,0,'Dealing Black Hand') ;

   BlackScore := 0 ;  BlackCounter := 0 ;
   REPEAT
      INC(BlackCounter) ;
      IF BlackScore = 0 THEN
         DrawCard(CardDeck[CardCounter].CardNum,
                  CardDeck[CardCounter].Suit,
                  BlackRow,CardCol[BlackCounter],7,FALSE)
      ELSE
         DrawCard(CardDeck[CardCounter].CardNum,
                  CardDeck[CardCounter].Suit,
                  BlackRow,CardCol[BlackCounter],7,TRUE) ;
      INC(BlackScore,CardVal(CardDeck[CardCounter].CardNum)) ;
      WriteBlackScore(BlackScore) ;
      INC(CardCounter) ;
      DELAY(DSpeed) ;
   UNTIL BlackScore >= 31 ;

   ClearMenu ;
   CursorPos(18,28) ;
   sDisplay(0,12,0,'Dealing Red Hand') ;

   RedScore := 0 ;  RedCounter := 0 ;
   REPEAT
      INC(RedCounter) ;
      IF RedScore = 0 THEN
         DrawCard(CardDeck[CardCounter].CardNum,
                  CardDeck[CardCounter].Suit,
                  RedRow,CardCol[RedCounter],7,FALSE)
      ELSE
         DrawCard(CardDeck[CardCounter].CardNum,
                  CardDeck[CardCounter].Suit,
                  RedRow,CardCol[RedCounter],7,TRUE) ;
      INC(RedScore,CardVal(CardDeck[CardCounter].CardNum)) ;
      WriteRedScore(RedScore) ;
      INC(CardCounter) ;
      DELAY(DSpeed) ;
   UNTIL RedScore >= 31 ;
   ClearMenu ;
   { check outcome }
   IF (RedScore = 31) AND (BlackScore = 31) THEN BEGIN
      WriteResult('Refait') ;
      WriteMsg('Refait (tie) at 31.  Bank takes half of your bet...') ;
      INC(Score,Half(BetAmount)) ;
      WriteScore ;
      EXIT ;
   END ELSE IF RedScore = BlackScore THEN BEGIN  { tie }
      WriteResult('Refait') ;
      WriteMsg('Refait (tie) at '+ ItoA(RedScore)+'.  All bets have been called off...') ;
      INC(Score,BetAmount) ;
      WriteScore ;
      EXIT ;
   END ELSE BEGIN
      GetDiffs ;  { get differences and sets blackdiff,reddiff }
      IF RedDiff < BlackDiff THEN BEGIN
         IF (FirstCard.Suit=D) OR (FirstCard.Suit=H) THEN BEGIN
            WriteResult('Couleur') ;
         END ELSE BEGIN
            WriteResult('Inverse') ;
         END ;
      END ELSE IF BlackDiff < RedDiff THEN BEGIN
         IF (FirstCard.Suit=C) OR (FirstCard.Suit=S) THEN BEGIN
            WriteResult('Couleur') ;
         END ELSE BEGIN
            WriteResult('Inverse') ;
         END ;
      END ;
      IF BetType IN ['R','B'] THEN BEGIN
         IF RedDiff < BlackDiff THEN BEGIN
            IF BetType = 'R' THEN BEGIN
               IF RedDiff = 0 THEN BEGIN
                  WriteMsg('Red wins with 31 on the money.  Paying off at 3 to 1...') ;
                  INC(Score,BetAmount*3) ;
               END ELSE BEGIN
                  WriteMsg('Red wins!  Paying off at 2 to 1...') ;
                  INC(Score,BetAmount*2) ;
               END ;
               WriteScore ;
            END ELSE BEGIN
               WriteMsg('Red wins.  You lose your bet...') ;
            END ;
         END ELSE IF BlackDiff < RedDiff THEN BEGIN
            IF BetType = 'B' THEN BEGIN
               IF BlackDiff = 0 THEN BEGIN
                  WriteMsg('Black wins with 31 on the money.  Paying off at 3 to 1...') ;
                  INC(Score,BetAmount*3) ;
               END ELSE BEGIN
                  WriteMsg('Black wins!  Paying off at 2 to 1...') ;
                  INC(Score,BetAmount*2) ;
               END ;
               WriteScore ;
            END ELSE BEGIN
               WriteMsg('Black wins.  You lose your bet...') ;
            END ;
         END ;
      END ELSE BEGIN  { player bet C or I }
         IF BetType = 'C' THEN BEGIN
            IF (FirstCard.Suit = C) OR (FirstCard.Suit = S) THEN BEGIN
               IF BlackDiff < RedDiff THEN BEGIN
                  WriteMsg('Couleur (color) wins!  Paying off at 3 to 1...') ;
                  INC(Score,BetAmount*3) ;
                  WriteScore ;
                  WriteResult('Couleur') ;
               END ELSE BEGIN
                  WriteMsg('Inverse wins.  You lose your bet...') ;
               END ;
            END ELSE BEGIN
               IF RedDiff < BlackDiff THEN BEGIN
                  WriteMsg('Couleur (color) wins!  Paying off at 3 to 1...') ;
                  INC(Score,BetAmount*3) ;
                  WriteScore ;
                  WriteResult('Couleur') ;
               END ELSE BEGIN
                  WriteMsg('Inverse wins.  You lose your bet...') ;
               END ;
            END ;
         END ELSE BEGIN
            IF (FirstCard.Suit = C) OR (FirstCard.Suit = S) THEN BEGIN
               IF BlackDiff > RedDiff THEN BEGIN
                  WriteMsg('Inverse wins!  Paying off at 3 to 1...') ;
                  INC(Score,BetAmount*3) ;
                  WriteScore ;
                  WriteResult('Inverse') ;
               END ELSE BEGIN
                  WriteMsg('Couleur (color) wins.  You lose your bet...') ;
                  WriteResult('Couleur') ;
               END ;
            END ELSE BEGIN
               IF RedDiff > BlackDiff THEN BEGIN
                  WriteMsg('Inverse wins!  Paying off at 3 to 1...') ;
                  INC(Score,BetAmount*3) ;
                  WriteScore ;
                  WriteResult('Inverse') ;
               END ELSE BEGIN
                  WriteMsg('Coluer (color) wins.  You lose your bet...') ;
                  WriteResult('Couleur') ;
               END ;
            END ;
         END ;
      END ;
   END ;
END ;



FUNCTION FixStr(n : INTEGER) : STRING ;
{ inserts a leading zero into a number
  between 0 and 9.  designed to fix
  date/time values.
}
VAR
   TVal    : STRING ;
BEGIN
   CASE n OF
      0..9 : TVal := '0' + ItoA(n) ;
      ELSE   TVal := ItoA(n) ;
   END ;
   FixStr := TVal ;
END ;

FUNCTION DaysAvailable : INTEGER ;
{ returns the number of days available for
  use as make-up days.
}
VAR
   Counter     : INTEGER ;
   jj          : INTEGER ;
BEGIN
   Counter := 0 ;
   IF dt = 1 THEN BEGIN                   { if global date var dt = 1 then   }
      DaysAvailable := 0 ;
      Exit ;
   END ;
   FOR jj := 1 TO (dt - 1) DO BEGIN
      IF PlayArray[jj] = '-' THEN INC(Counter) ;
   END ;
   DaysAvailable := Counter ;
END ;

FUNCTION MakeUpDay : BOOLEAN ;
VAR
   jj         : INTEGER ;
   Choice     : CHAR ;
   ii         : INTEGER ;
BEGIN
   ii := DaysAvailable ;
   ClearScreen ;
   sDisplay(0,15,0,'The System Operator allows ') ;  {line 1}
   sDisplay(0,14,0,ItoA(MaxMakeUpDays)) ;
   Display(0,15,0,' make up game(s) per day.') ;
   CRLF ;                                            {line 2}
   FOR jj := 1 TO 31 DO BEGIN
      IF jj <> dt THEN sDisplay(0,3,0,FixStr(jj)+'  ')
        ELSE sDisplay(0,14,0,FixStr(jj)+'  ') ;
      IF (jj=7) OR (jj=14) OR (jj=21) OR (jj=28) OR (jj=31) THEN BEGIN
         CRLF ;
         CRLF ;
      END ;
   END ;
   CursorPos(4,1) ;
   FOR jj := 1 TO 7 DO BEGIN
      IF (PlayArray[jj] = '-') AND (jj < dt) THEN sDisplay(0,14,0,'UU  ')
         ELSE IF PlayArray[jj] = 'X' THEN sDisplay(0,15,0,'XX  ')
         ELSE sDisplay(0,15,0,'--  ') ;
   END ;
   CursorPos(6,1) ;
   FOR jj := 8 TO 14 DO BEGIN
      IF (PlayArray[jj] = '-') AND (jj < dt) THEN sDisplay(0,14,0,'UU  ')
         ELSE IF PlayArray[jj] = 'X' THEN sDisplay(0,15,0,'XX  ')
         ELSE sDisplay(0,15,0,'--  ') ;
   END ;
   CursorPos(8,1) ;
   FOR jj := 15 TO 21 DO BEGIN
      IF (PlayArray[jj] = '-') AND (jj < dt) THEN sDisplay(0,14,0,'UU  ')
         ELSE IF PlayArray[jj] = 'X' THEN sDisplay(0,15,0,'XX  ')
         ELSE sDisplay(0,15,0,'--  ') ;
   END ;
   CursorPos(10,1) ;
   FOR jj := 22 TO 28 DO BEGIN
      IF (PlayArray[jj] = '-') AND (jj < dt) THEN sDisplay(0,14,0,'UU  ')
         ELSE IF PlayArray[jj] = 'X' THEN sDisplay(0,15,0,'XX  ')
         ELSE sDisplay(0,15,0,'--  ') ;
   END ;
   CursorPos(12,1) ;
   FOR jj := 29 TO 31 DO BEGIN
      IF (PlayArray[jj] = '-') AND (jj < dt) THEN sDisplay(0,14,0,'UU  ')
         ELSE IF PlayArray[jj] = 'X' THEN sDisplay(0,15,0,'XX  ')
         ELSE sDisplay(0,15,0,'--  ') ;
   END ;
   CRLF ; CRLF ;
   sDisplay(0,14,0,' UU') ; Display(0,7,0,' = Un-played day (available)') ;
   sDisplay(0,14,0,' XX') ; Display(0,7,0,' = Played day (not available)') ;
   sDisplay(0,14,0,' --') ; Display(0,7,0,' = Day is not playable') ;
   CRLF ;
   sDisplay(0,15,0,'You have used ') ;
   sDisplay(0,12,0,ItoA(UserMakeUps)) ;
   Display(0,15,0,' make up game(s) today.') ;
   sDisplay(0,15,0,'There are ') ;
   sDisplay(0,13,0,ItoA(ii)) ;
   Display(0,15,0,' available day(s) for make up games.') ;
   CRLF ;
   IF (ii > 0) AND (UserMakeUps < MaxMakeUpDays) THEN BEGIN
      sDisplay(0,7,0,'Select ') ;
      sDisplay(0,15,0,'M') ;
      sDisplay(0,7,0,'>ake up day, or ') ;
      sDisplay(0,15,0,'Q') ;
      sDisplay(0,7,0,'>uit game: ') ;
      REPEAT
         Choice := UPCASE(GetChar) ;
         IF ((UserMakeUps >= MaxMakeUpDays) OR (ii < 1))
         AND (Choice = 'M') THEN Choice := 'z' ;
      UNTIL Choice IN ['Q','M'] ;
      sDisplay(0,15,0,Choice) ;
   END ELSE BEGIN
      More('Press [any key] to continue',10) ;
      Choice := 'Q' ;
   END ;
   CASE Choice OF
      'Q' : BEGIN
               MakeUpDay := FALSE ;
               EXIT ;
               {
               ViewScores(1) ;
               HALT(0) ;       }
            END ;
      'M' : BEGIN
               FOR jj := 1 TO (dt - 1) DO BEGIN
                  IF PlayArray[jj] = '-' THEN BEGIN
                     PlayArray[jj] := 'X' ;
                     jj := dt - 1 ;
                  END ;
               END ;
               INC(UserMakeUps) ;
               TurnCount := 0 ;
               MakeUpDay := TRUE ;
               EXIT ;
            END ;
   END ;
END ;

PROCEDURE GetBetAmount ;
VAR
   MaxAvail     : LONGINT ;
   TVal         : STRING ;
BEGIN
   IF Score < MaxBet THEN MaxAvail := Score
      ELSE MaxAvail := MaxBet ;
   REPEAT
      WriteMsg('Enter amount to wager [$1...$'+ItoA(MaxAvail)+']: $') ;
      TVal := GetInput('',1,Length(ItoA(MaxAvail))) ;
      IF TVal = '' THEN BetAmount := 0 ELSE BetAmount := AtoI(TVal) ;
   UNTIL (BetAmount = 0) OR (BetAmount <= MaxAvail) ;
   IF BetAmount > 0 THEN BEGIN
      DEC(Score,BetAmount) ;
      WriteScore ;
      WriteBet ;
   END ;
END ;

PROCEDURE GetBetType ;
VAR
   Choice     : CHAR ;
BEGIN
   WriteMsg('Are you wagering on R>ed, B>lack, C>olor, or I>nverse: ') ;
   REPEAT
      Choice := UPCASE(GetChar) ;
   UNTIL Choice IN ['R','B','C','I'] ;
   sDisplay(0,14,0,Choice) ;
   BetType := Choice ;
END ;



PROCEDURE MenuAgent ;
VAR
   Choice     : CHAR ;
   TVal       : STRING[5] ;
   TNum       : INTEGER ;
BEGIN
   WriteMsg('Enter your selection...') ;
   REPEAT
      WriteMenu ;
      REPEAT
         Choice := UPCASE(GetChar) ;
      UNTIL Choice IN ['D','H','P','V','Q'] ;
      sDisplay(0,14,0,Choice) ;
      CASE Choice OF
         'P' : BEGIN
                  IF TurnCount >= MaxTurns THEN BEGIN
                     WriteMsg('Your turn is over for today.') ;
                     JPDelay(2,FALSE,FALSE) ;
                  END ELSE IF Score < 1 THEN BEGIN
                     WriteMsg('You have no money to gamble. Come back again tomorrow.') ;
                     JPDelay(2,FALSE,FALSE) ;
                  END ELSE BEGIN
                     IF NOT FirstPlay THEN BEGIN
                        ClearBlackScore ;
                        ClearRedScore ;
                        ClearCards ;
                        ClearBet ;
                        ClearFirstCard ;
                        ClearResult ;
                     END ELSE FirstPlay := FALSE ;
                     PlayArray[dt] := 'X' ;
                     INC(TurnCount) ;
                     WriteTurnCount ;
                     ShuffleDeck ;
                     GetBetAmount ;
                     IF BetAmount > 0 THEN BEGIN
                        GetBetType ;
                        DealCards ;
                     END ELSE WriteMsg('Enter your selection...') ;
                  END ;
               END ;
         'H' : BEGIN
                  ClearScreen ;
                  ScrLen := 23 ;
                  AscFile('Rob.Hlp') ;
                  CRLF ;
                  More('Press [any key] to continue',15) ;
                  IF ANSI = 1 THEN GameScreen('Rob.ANS')
                   ELSE GameScreen('Rob.ASC') ;
                  WriteMaxTurns ;
                  WriteTurnCount ;
                  WriteScore ;
                  FirstPlay := TRUE ;
               END ;
         'V' : BEGIN
                  ClearScreen ;
                  ScrLen := 23 ;
                  ViewScores(2) ;
                  IF ANSI = 1 THEN GameScreen('Rob.ANS')
                   ELSE GameScreen('Rob.ASC') ;
                  WriteMaxTurns ;
                  WriteTurnCount ;
                  WriteScore ;
                  FirstPlay := TRUE ;
               END ;
         'D' : BEGIN
                  WriteMsg('Delay speed ('+ItoA(DSpeed)+'): 50=Fast, 1500=Slow, <ENTER>=No Change [50..1500]: ') ;
                  TVal := GetInput('',1,4) ;
                  TNum := AtoI(TVal) ;
                  IF (TNum > 49) AND (TNum < 1501) THEN DSpeed := TNum ;
                  WriteMsg('Delay speed: ' + ItoA(DSpeed)+'.') ;
                  sDisplay(0,12,0,'  Press <ENTER>') ;
                  Holding := GetChar ;
                  WriteMsg('Enter your selection...') ;
               END ;
         'Q' : BEGIN
                  IF TurnCount >= MaxTurns THEN BEGIN
                     IF MakeUpDay THEN BEGIN
                        Choice := 'z' ;
                        IF ANSI = 1 THEN GameScreen('Rob.ANS')
                          ELSE GameScreen('Rob.ASC') ;
                        WriteTurnCount ;
                        WriteMaxTurns ;
                        WriteScore ;
                     END ;
                  END ;
               END ;
      END ;
   UNTIL Choice = 'Q' ;
END ;

PROCEDURE SetClub ;
VAR
   Choice    : CHAR ;
BEGIN
   CRLF ;
   Display(0,7,0,'Some communications programs do not display the CLUB character') ;
   Display(0,7,0,'properly.  If you are using one of these communications programs,') ;
   Display(0,7,0,'the CLUB character will not be visible in the prompt below.  If') ;
   Display(0,7,0,'this is the case, then answer No to the prompt.') ;
   CRLF ;
   sDisplay(0,15,0,'Do you see a CLUB character between the arrows? ') ;
   sDisplay(0,14,0,'> ') ;
   sDisplay(0,12,0,ClubChar) ;
   sDisplay(0,14,0,' <') ;
   sDisplay(0,15,0,' (Y,n): ') ;
   REPEAT
      Choice := UPCASE(GetChar) ;
      IF Choice = #13 THEN Choice := 'Y' ;
   UNTIL Choice IN ['Y','N'] ;
   sDisplay(0,15,0,Choice) ;
   IF Choice = 'N' THEN ClubChar := 'C' ;
END ;


PROCEDURE InitVars ;
BEGIN
   FileMode := 66;
   CheckBreak := TRUE ;
   ASSIGN(output,'') ;
   REWRITE(output) ;
   ProductName := 'Red Or Black ' + Version ;
   CkExitInfo := FALSE ;
   CkMsgs := FALSE ;
   DoorExit := TrapExit ;
   Score := 0 ;
   TurnCount := 0 ;
   UserName := '<not yet known>' ;
   AbortGame := FALSE ;
   FirstPlay := TRUE ;
   CardCounter := 0 ;
   DSpeed := 1000 ;
END ;


FUNCTION GetUserVals : INTEGER ;
VAR
   j         : BYTE ;
   UserVal1,
   UserVal2  : INTEGER ;
BEGIN
   UserVal1 := 0 ; UserVal2 := 0 ;
   FOR j := 1 TO LENGTH(SysopName) DO BEGIN
      UserVal1 := UserVal1 + ORD(SysopName[j]) ;
   END ;
   FOR j := 1 TO LENGTH(SystemName) DO BEGIN
      UserVal2 := UserVal2 + ORD(SystemName[j]) ;
   END ;
   GetUserVals := UserVal1 + UserVal2 ;
   EXIT ;
END ;


BEGIN          (* M A I N *)
   GetDate(yr,mn,dt,dy) ;
   GameStarted := FALSE ;
   InitVars ;
   GetParms ;
   GetDorInfo('1',DropPath) ;
   UserName := ChangeCase(UserName) ;
   SysopName := ChangeCase(SysopName) ;
   SystemName := ChangeCase(SystemName) ;
   SysopKey[1] := #0 + #46 ;
{   IF NOT SetFossil THEN WRITELN('Could not initialize fossil driver!')
     ELSE WRITELN('Fossil initialized') ;  }
   SetColor(0,7,0) ;
   Graphics := SetGraphics ;
   IF Graphics = 'A' THEN BEGIN
      CRLF ;
      Display(0,15,0,'Due to the graphics requirement of this door, a minimum setting') ;
      Display(0,15,0,'of "Mono graphics" must be used.') ;
      CRLF ;
      HALT(8) ;
   END ;
   SetClub ;
   StatusLineOn := TRUE ;
   Show_Status ;

   ClearScreen ;
   Crlf ; Crlf ;
   Display(0,15,0,Center('Red Or Black version ' + Version,79)) ;
   Display(0,15,0,Center('Copyright (C) 1991 by John Parlin',79)) ;
   Display(0,15,0,Center('All rights reserved',79)) ;
   CRLF ;

   sDisplay(0,9,1,Center('UNREGISTERED EVALUATION',79)) ;
   Display(0,7,0,' ') ;
   CRLF ;
   Display(0,15,0,Center('You may play '+ItoA(MaxTurns)+
                  ' game(s) per day.',79)) ;
   IF EXIST('Rob.HSC') THEN BEGIN
      ASSIGN(HighFile,'Rob.HSC') ;
      RESET(HighFile) ;
      CRLF ;
      sDisplay(1,15,0,' LAST MONTH''S TOP THREE SCORERS:                                              ') ;
      sDisplay(0,7,0,' ') ;
      CRLF ; CRLF ;
      Display(0,13,0,MkString(17,' ')+ 'Player Name                         Score') ;
      Display(0,7,0,MkString(17,' ') + '---------------------------------   ----------') ;
      WHILE NOT EOF(HighFile) DO BEGIN
         ReadHighScore;
         sDisplay(0,11,0,MkString(17,' ')+TabStr(FormatStr(High.Playername),36)) ;
         Display(0,11,0,Stri(High.Score,10)) ;
      END ;
      CLOSE(HighFile) ;
   END ;
   CRLF ;
   sDisplay(0,12,0,Center('Press [any key] to continue',79)) ;
   Holding := GetChar ;
   ClearScreen ;
   SetHotKeys_ON(NoKill,['S']) ;
   ShowFile(15,0,0,'RobOpen','') ;
   SetHotKeys_OFF ;
   SetColor(0,7,0) ; sDisplay(0,7,0,' ') ;
   holding := getchar ;
   CheckMonth ;
   GetUserInfo ;
   GameStarted := TRUE ;
   IF TurnCount >= MaxTurns THEN ShowMaxTurns ELSE BEGIN
      IF ANSI = 1 THEN GameScreen('Rob.ANS')
        ELSE GameScreen('Rob.ASC') ;
      WriteTurnCount ;
      WriteMaxTurns ;
      WriteScore ;
      MenuAgent ;
   END ;
   ViewScores(1) ;
   HALT(0) ;
END.


